package dbrown;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.geom.Ellipse2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JComponent;

/**
 * A GUI interface that shows how many guesses a user has made in the form of a
 * hanged man. The hanged man has six pieces - a head, a trunk, two arms, and
 * two legs. As the player makes a wrong guess a new body part is added to the
 * hanged man. When the man is complete the game is over. Thus the number of
 * wrong guesses in the Hangman game must match the number of body parts on the
 * hanged man. The view dynamically scales itself to fit the game window. There
 * are no listeners for this view as there is no user input possible in it.
 *
 * @author David Brown (from Byron Weber Becker)
 * @version 2011-11-11
 */
@SuppressWarnings("serial")
public class GallowsView extends JComponent {

    /**
     * Is called by the Hangman game to display the current state of the game
     * when the game is started and whenever a guess is made.
     * <code>repaint</code> calls <code>paintComponent</code> as part of its
     * execution.
     */
    private class GameListener implements PropertyChangeListener {

	@Override
	public void propertyChange(final PropertyChangeEvent evt) {
	    GallowsView.this.repaint();
	}
    }

    // -------------------------------------------------------------------------------

    /**
     * Various colour constants defined for the view.
     */
    private static final Color GALLOWS_COLOR = new Color(100, 100, 100);
    private static final Color GRASS_COLOR = new Color(50, 175, 50);
    private static final Color MOUNTAIN_COLOR = new Color(20, 100, 20);
    private static final Color PERSON_COLOR = Color.WHITE;
    private static final Color SKY_COLOR = new Color(240, 240, 255);
    private static final Color SUN_COLOR = new Color(200, 100, 100);

    /**
     * The path definition for the body of the hanged man.
     */
    private final GeneralPath body = new GeneralPath();

    /**
     * The path definition for the gallows.
     */
    private final GeneralPath gallows = new GeneralPath();

    /**
     * The rectangle definition for the grass.
     */
    private final Rectangle2D.Double grass = new Rectangle2D.Double();

    /**
     * The ellipse definition for the head of the hanged man.
     */
    private final Ellipse2D.Double head = new Ellipse2D.Double();

    /**
     * The x-axis scaling factor for the graphical display.
     */
    private int hUnit;

    /**
     * The polygon definition for the mountain.
     */
    private final Polygon mountain = new Polygon();

    /**
     * The rectangle definition for the sky.
     */
    private final Rectangle2D.Double sky = new Rectangle2D.Double();

    /**
     * The ellipse definition for the sun.
     */
    private final Ellipse2D.Double sun = new Ellipse2D.Double();

    /**
     * The y-axis scaling factor for the graphical display.
     */
    private int wUnit;

    /**
     * The Hangman game object.
     */
    private final Hangman model;

    /**
     * 
     */
    public GallowsView(final Hangman model) {
	this.model = model;
	this.registerListeners();
    }

    /**
     * Draws the gallows to the view.
     *
     * @param g
     *            - the <code>Graphics2D</code> object used for drawing
     */
    public void drawGallows(final Graphics2D g) {
	this.gallows.reset();
	this.gallows.moveTo(2 * this.wUnit, 10 * this.hUnit);
	this.gallows.lineTo(8 * this.wUnit, 10 * this.hUnit);
	this.gallows.moveTo(7 * this.wUnit, 10 * this.hUnit);
	this.gallows.lineTo(7 * this.wUnit, 3 * this.hUnit);
	this.gallows.lineTo(5 * this.wUnit, 3 * this.hUnit);
	this.gallows.lineTo(5 * this.wUnit, 3.5 * this.hUnit);
	g.setStroke(new BasicStroke(this.wUnit / 10));
	g.setColor(GallowsView.GALLOWS_COLOR);
	g.draw(this.gallows);
    }

    /*
     * Overrides the standard paint method for the JComponent.
     */
    @Override
    public void paintComponent(final Graphics g) {
	super.paintComponent(g);
	final Graphics2D g2d = (Graphics2D) g;
	// Calculate the scaling factors.
	this.wUnit = this.getSize().width / 10;
	this.hUnit = this.getSize().height / 12;
	this.drawBackground(g2d);
	this.drawGallows(g2d);
	this.drawPerson(g2d);
    }

    /**
     * Draws the grass, sun, sky, and mountain background to the view.
     *
     * @param g
     *            - the <code>Graphics2D</code> object used for drawing
     */
    private void drawBackground(final Graphics2D g) {
	// Define and draw the sky. (Fills entire panel.)
	g.setColor(GallowsView.SKY_COLOR);
	this.sky.setRect(0, 0, this.getSize().width, this.getSize().height);
	g.fill(this.sky);
	// Define and draw the sun.
	this.sun.setFrame(3 * this.wUnit, 2 * this.hUnit, 2 * this.wUnit,
		2 * this.wUnit);
	g.setColor(GallowsView.SUN_COLOR);
	g.fill(this.sun);
	// Define and draw the grass.
	this.grass.setRect(0, 7 * this.hUnit, this.getSize().width,
		this.getSize().height - 7 * this.hUnit);
	g.setColor(GallowsView.GRASS_COLOR);
	g.fill(this.grass);
	// Define and draw the mountain.
	g.setColor(GallowsView.MOUNTAIN_COLOR);
	this.mountain.reset();
	this.mountain.addPoint(1 * this.wUnit, 7 * this.hUnit);
	this.mountain.addPoint(3 * this.wUnit, 2 * this.hUnit);
	this.mountain.addPoint(4 * this.wUnit, 3 * this.hUnit);
	this.mountain.addPoint(5 * this.wUnit, 1 * this.hUnit);
	this.mountain.addPoint(8 * this.wUnit, 7 * this.hUnit);
	g.fill(this.mountain);
    }

    /**
     * Draws the hanged man segment by segment depending on the number of wrong
     * guesses made by the player.
     *
     * @param g
     *            - the <code>Graphics2D</code> object used for drawing
     */
    private void drawPerson(final Graphics2D g) {
	g.setColor(GallowsView.PERSON_COLOR);
	final double x0 = 5.0 * this.wUnit;
	g.setStroke(new BasicStroke(this.wUnit / 5, BasicStroke.CAP_ROUND,
		BasicStroke.JOIN_ROUND));
	// Define the body position.
	this.body.reset();
	this.body.moveTo(x0, 4.5 * this.hUnit);

	// Draw the body by cascading through the number of wrong guesses.
	final int segments = this.model.getWrongGuesses();

	if (segments >= 1) {
	    // Draw the head.
	    this.head.setFrame(4.5 * this.wUnit, 3.5 * this.hUnit, this.wUnit,
		    this.hUnit);
	    g.fill(this.head);
	}
	if (segments >= 2) {
	    // Define the trunk.
	    this.body.lineTo(x0, 7 * this.hUnit);
	}
	if (segments >= 3) {
	    // Define the left arm.
	    this.body.moveTo(x0, 5 * this.hUnit);
	    this.body.lineTo(x0 - this.wUnit, 6 * this.hUnit);
	}
	if (segments >= 4) {
	    // Define the right arm.
	    this.body.moveTo(x0, 5 * this.hUnit);
	    this.body.lineTo(x0 + this.wUnit, 6 * this.hUnit);
	}
	if (segments >= 5) {
	    // Define the left leg.
	    this.body.moveTo(x0, 7 * this.hUnit);
	    this.body.lineTo(x0 - this.wUnit, 8 * this.hUnit);
	}
	if (segments >= 6) {
	    // Define the right leg.
	    this.body.moveTo(x0, 7 * this.hUnit);
	    this.body.lineTo(x0 + this.wUnit, 8 * this.hUnit);
	}
	if (segments > 1) {
	    // Draw the body as appropriate.
	    g.draw(this.body);
	}
    }

    /**
     * 
     */
    private void registerListeners() {
	this.model.addPropertyChangeListener(new GameListener());
    }
}
