package dbrown;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

/**
 * A GUI interface to part of the Hangman game. This illustrates how a model (in
 * this case the Hangman game) can be viewed by different interfaces. This view
 * displays only the game control buttons new game and forfeit. These buttons
 * are enabled / disabled depending on the state of the game.
 *
 * @author David Brown (from Byron Weber Becker)
 * @version 2011-11-11
 */
@SuppressWarnings("serial")
public class GameControlView extends JPanel {

    /**
     * Ends the game when the <code>forfeit</code> button is pressed.
     */
    private class ForfeitListener implements ActionListener {

	@Override
	public void actionPerformed(final ActionEvent e) {
	    GameControlView.this.forfeit.setEnabled(false);
	    GameControlView.this.newGame.setEnabled(true);
	    GameControlView.this.model.forfeit();
	}
    }

    // -------------------------------------------------------------------------------

    /**
     * Displays a message when the game is lost.
     */
    private class LoseListener implements PropertyChangeListener {

	@Override
	public void propertyChange(final PropertyChangeEvent evt) {
	    GameControlView.this.forfeit.setEnabled(false);
	    GameControlView.this.newGame.setEnabled(true);
	    JOptionPane.showMessageDialog(
		    GameControlView.this.getTopLevelAncestor(),
		    "Sorry, you have lost.");
	}
    }

    // -------------------------------------------------------------------------------

    /**
     * Starts a new game when the <code>newGame</code> button is pressed.
     */
    private class NewGameListener implements ActionListener {

	@Override
	public void actionPerformed(final ActionEvent e) {
	    GameControlView.this.forfeit.setEnabled(true);
	    GameControlView.this.newGame.setEnabled(false);
	    GameControlView.this.model.newGame();
	}
    }

    // -------------------------------------------------------------------------------

    /**
     * Displays a message when the game is won.
     */
    private class WinListener implements PropertyChangeListener {

	@Override
	public void propertyChange(final PropertyChangeEvent evt) {
	    GameControlView.this.forfeit.setEnabled(false);
	    GameControlView.this.newGame.setEnabled(true);
	    JOptionPane.showMessageDialog(
		    GameControlView.this.getTopLevelAncestor(),
		    "Congratulations, you have won!");
	}
    }

    // -------------------------------------------------------------------------------

    /**
     * The Hangman game object.
     */
    private final Hangman model;

    // Buttons
    private final JButton forfeit = new JButton(Hangman.FORFEIT);
    private final JButton newGame = new JButton(Hangman.NEW_GAME);

    /**
     * @param model
     *            - the Hangman game object
     */
    public GameControlView(final Hangman model) {
	super();
	this.model = model;
	this.layoutView();
	this.registerListeners();
    }

    /**
     * Lays out the view.
     */
    private void layoutView() {
	this.setLayout(new GridLayout(2, 1));
	this.add(this.forfeit);
	this.add(this.newGame);
    }

    /**
     * Assigns listeners to the view buttons and the model.
     */
    private void registerListeners() {
	// Add action listeners to the buttons.
	this.forfeit.addActionListener(new ForfeitListener());
	this.newGame.addActionListener(new NewGameListener());
	// Add property change listeners to the model.
	this.model.addPropertyChangeListener(Hangman.LOSE, new LoseListener());
	this.model.addPropertyChangeListener(Hangman.WIN, new WinListener());
    }

}
