package dbrown;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.FileNotFoundException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * The main Hangman game class. It provides the attributes and methods for
 * playing the game.
 *
 * @author David Brown (from Byron Weber Becker)
 * @version 2011-11-11
 */
public class Hangman {
    /**
     * The possible states of the game.
     */
    public static enum GameStateFlags {
	LOST, NEW, PLAYING, WON
    }

    // Constants
    public static final String FORFEIT = "Forfeit";
    public static final String NEW_GAME = "New Game";
    public static final String WIN = "Win";
    public static final String LOSE = "Lose";

    public static final String GUESS = "Guess";

    /**
     * The end points of the valid phrase letters.
     */
    public static final char FIRST_LETTER = 'A';
    public static final char LAST_LETTER = 'Z';

    /**
     * The maximum number of allowed wrong guesses. This must match the number
     * of body parts in a gallows view.
     */
    public static final int MAX_WRONG_GUESSES = 6;

    /**
     * The number of letters in the alphabet.
     */
    public static final int NUM_LETTERS = 26;

    /**
     * Regular expressions are used to convert the phrase to a string of
     * asterisks. This pattern excludes white space characters from the
     * characters to be converted.
     */
    public static final Pattern pattern = Pattern.compile("\\S");

    /**
     * Contains the current state of the game.
     */
    private GameStateFlags gameState = Hangman.GameStateFlags.NEW;
    /**
     * Letters of the alphabet may be guessed or not yet guessed and their
     * states are contained in this array.
     */
    private final boolean[] guessedLetters = new boolean[Hangman.NUM_LETTERS];

    /**
     * The phrase guessed so far is stored in a mutable structure.
     */
    private StringBuffer guessedPhrase = new StringBuffer(" ");

    /**
     * The phrase to be guessed.
     */
    private String phrase = " ";

    /**
     * The list of phrases to be guessed.
     */
    private PhraseList phrases = null;

    /**
     * The number of wrong guesses made so far.
     */
    private int wrongGuesses = 0;

    /**
     * Allows views to listen to generic or named changes in the hangman game.
     */
    private final PropertyChangeSupport pcs = new PropertyChangeSupport(this);

    /**
     * @param filename
     *            - the name of the file to open that contains the phrases to be
     *            guessed
     */
    public Hangman(final String filename) throws FileNotFoundException {
	this.phrases = new PhraseList(filename);
	this.initGuessedLetters(true);
    }

    /**
     * Allows for generic listeners. Listeners will update no matter what change
     * has taken place in the Hangman model.
     *
     * @param listener
     */
    public void addPropertyChangeListener(
	    final PropertyChangeListener listener) {
	this.pcs.addPropertyChangeListener(listener);
    }

    /**
     * Allows for named listeners. Listeners will update only when a named
     * property of the Hangman model is updated.
     *
     * @param propertyName
     * @param listener
     */
    public void addPropertyChangeListener(final String propertyName,
	    final PropertyChangeListener listener) {
	this.pcs.addPropertyChangeListener(propertyName, listener);
    }

    /**
     * Forfeits the game and exposes the phrase to be guessed.
     */
    public void forfeit() {
	this.guessedPhrase = new StringBuffer(this.phrase);
	this.initGuessedLetters(true);
	this.gameState = Hangman.GameStateFlags.LOST;
	this.pcs.firePropertyChange(FORFEIT, null, null);
    }

    /**
     * @return the phrase as guessed so far
     */
    public String getGuessedPhrase() {
	return new String(this.guessedPhrase);
    }

    /**
     * @return the number of wrong guesses so far
     */
    public int getWrongGuesses() {
	return this.wrongGuesses;
    }

    /**
     * Accepts a letter guessed by the player and checks to see whether it has
     * already been guessed and whether is is part of the phrase to be guessed.
     * It further determines whether the game has been won, lost, or should
     * continue.
     *
     * @param letter
     *            - the letter guessed by the player
     * @throws Exception
     *             - when the letter has already been guessed
     */
    public void guess(final char letter) throws Exception {
	if (this.gameState == Hangman.GameStateFlags.PLAYING) {
	    boolean goodGuess = false;

	    if (this.wasGuessed(letter)) {
		throw new Exception("" + letter + " has already been guessed.");
	    }
	    // Looks for multiple occurrences of the guessed letter in the
	    // target phrase.
	    int i = this.phrase.indexOf(letter);

	    while (i >= 0) {
		goodGuess = true;
		this.guessedPhrase.setCharAt(i, letter);
		i = this.phrase.indexOf(letter, i + 1);
	    }
	    // Increments the number of wrong guesses if the letter is not found
	    // in the target phrase.
	    if (!goodGuess) {
		this.wrongGuesses++;
	    }
	    // Updates the list of guessed letters.
	    this.guessedLetters[letter - Hangman.FIRST_LETTER] = true;

	    // Inform listeners a guess has been made.
	    this.pcs.firePropertyChange(GUESS, null, null);

	    // Determine the state of the game.
	    if (this.wrongGuesses >= Hangman.MAX_WRONG_GUESSES) {
		this.guessedPhrase = new StringBuffer(this.phrase);
		this.initGuessedLetters(true);
		this.gameState = Hangman.GameStateFlags.LOST;
		// Inform listeners the game is lost.
		this.pcs.firePropertyChange(LOSE, null, null);
	    } else if (this.phrase.equals(this.guessedPhrase.toString())) {
		this.initGuessedLetters(true);
		this.gameState = Hangman.GameStateFlags.WON;
		// Inform listeners the game is won.
		this.pcs.firePropertyChange(WIN, null, null);
	    }
	}
    }

    /**
     * @return whether the game is currently being played
     */
    public boolean isPlaying() {
	return this.gameState == Hangman.GameStateFlags.PLAYING;
    }

    /**
     * @return whether the player has lost the game
     */
    public boolean lost() {
	return this.gameState == Hangman.GameStateFlags.LOST;
    }

    /**
     * Initializes a new game by selecting a phrase from the file of phrases in
     * the <code>phraseList</code> object.
     */
    public void newGame() {
	this.newGame(this.phrases.randomPhrase());
	this.pcs.firePropertyChange(NEW_GAME, null, null);
    }

    /**
     * Initializes a new game with a given phrase.
     *
     * @param newPhrase
     *            the phrase for the player to guess.
     */
    public void newGame(final String newPhrase) {
	this.phrase = newPhrase.toUpperCase();
	// Uses regular expressions to convert the phrase to a list of
	// asterisks. White space is left alone.
	final Matcher m = Hangman.pattern.matcher(this.phrase);
	this.guessedPhrase = new StringBuffer(m.replaceAll("*"));
	// Reset the list of guessed letters, the number of guesses made and the
	// game state.
	this.initGuessedLetters(false);
	this.wrongGuesses = 0;
	this.gameState = Hangman.GameStateFlags.PLAYING;
	// Update all views of the game.
	this.pcs.firePropertyChange(NEW_GAME, null, null);
    }

    /**
     * Determines whether a character guessed by the player has already been
     * guessed or not.
     *
     * @param letter
     *            the letter the player has guessed as being part of the current
     *            phrase
     * @return whether the letter has already been guessed or not
     * @throws IllegalArgumentException
     *             when the character passed as a parameter is not part of the
     *             alphabet
     */
    public boolean wasGuessed(final char letter)
	    throws IllegalArgumentException {
	if (letter < Hangman.FIRST_LETTER || letter > Hangman.LAST_LETTER) {
	    throw new IllegalArgumentException("Character " + letter
		    + " out of range. Must be between " + Hangman.FIRST_LETTER
		    + " and " + Hangman.LAST_LETTER + ".");
	}
	return this.guessedLetters[letter - Hangman.FIRST_LETTER];
    }

    /**
     * @return whether the player has won the game
     */
    public boolean won() {
	return this.gameState == Hangman.GameStateFlags.WON;
    }

    /**
     * Initializes the list of guessed letters to not guessed so far.
     */
    private void initGuessedLetters(final boolean state) {
	for (int i = 0; i < this.guessedLetters.length; i++) {
	    this.guessedLetters[i] = state;
	}
    }
}
