package dbrown;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JOptionPane;
import javax.swing.JPanel;

/**
 * A GUI interface to part of the Hangman game. This illustrates how a model (in
 * this case the Hangman game) can be viewed by different interfaces. This view
 * displays the letter buttons available to the user when guessing the hidden
 * phrase.
 *
 * @author David Brown (from Byron Weber Becker)
 * @version 2011-11-11
 */
@SuppressWarnings("serial")
public class LettersView extends JPanel {

    /**
     * A private inner class that calls the Hangman <code>guess</code> method
     * with the appropriate letter when the assigned button is pressed.
     */
    private class ButtonListener implements ActionListener {

	@Override
	public void actionPerformed(final ActionEvent e) {
	    try {
		LettersView.this.model
			.guess(((LetterButton) e.getSource()).getLetter());
	    } catch (final Exception ex) {
		// Displays an error message if the selected letter has already
		// been guessed - should never actually be caught in a properly
		// designed GUI.
		JOptionPane.showMessageDialog(
			LettersView.this.getTopLevelAncestor(),
			ex.getMessage());
	    }
	}
    }

    // -------------------------------------------------------------------------------

    /**
     * Is called by the Hangman game to display the current state of the game
     * when the game is started and whenever a guess is made.
     *
     */
    private class GuessListener implements PropertyChangeListener {

	@Override
	public void propertyChange(final PropertyChangeEvent evt) {
	    // Sets the enabled state of every button from the guessed state of
	    // each letter in the game. Already guessed letters disable their
	    // matching button.
	    for (final LetterButton button : LettersView.this.buttons) {
		final char ch = button.getLetter();
		button.setEnabled(!LettersView.this.model.wasGuessed(ch));
	    }
	}
    }

    // -------------------------------------------------------------------------------

    /**
     * An array of buttons, each of which is associated with a letter of the
     * alphabet.
     */
    private final LetterButton[] buttons = new LetterButton[Hangman.NUM_LETTERS];

    /**
     * The Hangman game object.
     */
    private final Hangman model;

    /**
     * @param model
     *            - the Hangman game object
     */
    public LettersView(final Hangman model) {
	this.model = model;
	int i = 0;

	// Instantiates a new letter button with a unique letter for every
	// element of the letter button array.
	for (char ch = Hangman.FIRST_LETTER; ch <= Hangman.LAST_LETTER; ch++) {
	    this.buttons[i] = new LetterButton(ch);
	    i++;
	}
	this.layoutView();
	this.registerListeners();
    }

    /**
     * Adds the letter buttons to the display panel in a grid format.
     */
    private void layoutView() {
	this.setLayout(new GridLayout(13, 2));

	for (final LetterButton button : this.buttons) {
	    this.add(button);
	}
    }

    /**
     * Adds a button listener to each button in the view. Adds a property
     * listener to listen to changes in the game state.
     */
    private void registerListeners() {
	final ButtonListener listener = new ButtonListener();

	for (final LetterButton button : this.buttons) {
	    button.addActionListener(listener);
	}
	this.model.addPropertyChangeListener(new GuessListener());
    }

}
