package dbrown;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Scanner;

/**
 * A simple text interface to the Hangman game. This illustrates how a model (in
 * this case the Hangman game) can be viewed by different interfaces. This is a
 * text-only view of the model, whereas other views are GUI based.
 *
 * @author David Brown
 * @version 2011-11-11
 */
public class TextView {

    // -------------------------------------------------------------------------------
    /**
     * Is called by the Hangman model to display the current state of the model
     * when the model is started and whenever a guess is made.
     */
    private class GameListener implements PropertyChangeListener {

	@Override
	public void propertyChange(final PropertyChangeEvent evt) {
	    System.out.println(
		    "Phrase so far: " + TextView.this.model.getGuessedPhrase());
	    System.out.println("Guesses left: " + (Hangman.MAX_WRONG_GUESSES
		    - TextView.this.model.getWrongGuesses()));
	}
    }

    // -------------------------------------------------------------------------------

    /**
     * Displays a message when the game is lost.
     */
    private class LoseListener implements PropertyChangeListener {

	@Override
	public void propertyChange(final PropertyChangeEvent evt) {
	    // Tell the user they have lost.
	    System.out.println();
	    System.out.println(
		    "Phrase: " + TextView.this.model.getGuessedPhrase());
	    System.out.println("Sorry, you lost.");
	}
    }

    // -------------------------------------------------------------------------------

    /**
     * Displays a message when the game is won.
     */
    private class WinListener implements PropertyChangeListener {

	@Override
	public void propertyChange(final PropertyChangeEvent evt) {
	    // Tell the user they have won.
	    System.out.println();
	    System.out.println(
		    "Phrase: " + TextView.this.model.getGuessedPhrase());
	    System.out.println("Congratulations, you won!");
	}
    }

    // -------------------------------------------------------------------------------
    /**
     * The Hangman game object.
     */
    private Hangman model = null;

    /**
     * @param model
     */
    public TextView(final Hangman model) {
	this.model = model;
	this.registerListeners();
    }

    /**
     * Play the Hangman game by instantiating the game and asking the user for
     * input. Allows only a single game to be played per run.
     */
    public void play() {
	System.out.println("Welcome to Hangman!");
	// Start a new game.
	this.model.newGame();

	// Set up the keyboard interface.
	char letter;
	final Scanner s = new Scanner(System.in);
	System.out.println("Welcome to hangman!");

	while (this.model.isPlaying()) {
	    try {
		// Read a single character at a time from the keyboard and
		// pass to the model as a guess.
		System.out.print("Enter a guess: ");
		letter = s.next().toUpperCase().charAt(0);
		this.model.guess(letter);
	    } catch (final Exception e) {
		// Flag input errors, but continue the model.
		System.err.println("Error: " + e.getMessage());
		System.out.println();
	    }
	}
	s.close();
    }

    /**
     * Registers model property listeners.
     */
    private void registerListeners() {
	this.model.addPropertyChangeListener(new GameListener());
	this.model.addPropertyChangeListener(Hangman.LOSE, new LoseListener());
	this.model.addPropertyChangeListener(Hangman.WIN, new WinListener());
    }

}
